import os
from .transcribe import Transcribe, TranscriptionOptions
from .languages import from_language_to_iso_code
import numpy as np
import warnings
from typing import Union, List
from .writers import get_writer
from .live import Live
import sys
import datetime
from .commandline import CommandLine


def get_diarization(audio, diarize_model, verbose):
    diarization_output = {}
    for audio_path in audio:
        if verbose and len(audio) > 1:
            print(f"\nFile: '{audio_path}' (diarization)")

        start_time = datetime.datetime.now()
        diarize_segments = diarize_model.run_model(audio_path)
        diarization_output[audio_path] = diarize_segments
        if verbose:
            print(f"Time used for diarization: {datetime.datetime.now() - start_time}")

    diarize_model.unload_model()
    return diarization_output


def get_transcription_options(args):
    temperature = args.pop("temperature")

    if (increment := args.pop("temperature_increment_on_fallback")) is not None:
        temperature = tuple(np.arange(temperature, 1.0 + 1e-6, increment))
    else:
        temperature = [temperature]

    suppress_tokens: str = args.pop("suppress_tokens")

    if suppress_tokens is None or len(suppress_tokens) == 0:
        suppress_tokens = []
    else:
        suppress_tokens = [int(t) for t in suppress_tokens.split(",")]

    return TranscriptionOptions(
        beam_size=args.pop("beam_size"),
        best_of=args.pop("best_of"),
        patience=args.pop("patience"),
        length_penalty=args.pop("length_penalty"),
        repetition_penalty=args.pop("repetition_penalty"),
        no_repeat_ngram_size=args.pop("no_repeat_ngram_size"),
        log_prob_threshold=args.pop("logprob_threshold"),
        no_speech_threshold=args.pop("no_speech_threshold"),
        compression_ratio_threshold=args.pop("compression_ratio_threshold"),
        condition_on_previous_text=args.pop("condition_on_previous_text"),
        temperature=temperature,
        prompt_reset_on_temperature=args.pop("prompt_reset_on_temperature"),
        initial_prompt=args.pop("initial_prompt"),
        suppress_blank=args.pop("suppress_blank"),
        suppress_tokens=suppress_tokens,
        word_timestamps=args.pop("word_timestamps"),
        prepend_punctuations=args.pop("prepend_punctuations"),
        append_punctuations=args.pop("append_punctuations"),
        print_colors=args.pop("print_colors"),
        hallucination_silence_threshold=args.pop("hallucination_silence_threshold"),
        vad_filter=args.pop("vad_filter"),
        vad_threshold=args.pop("vad_threshold"),
        vad_min_speech_duration_ms=args.pop("vad_min_speech_duration_ms"),
        vad_max_speech_duration_s=args.pop("vad_max_speech_duration_s"),
        vad_min_silence_duration_ms=args.pop("vad_min_silence_duration_ms"),
    )


def get_language(language, model_directory, model):
    language = from_language_to_iso_code(language)

    if (
        not model_directory
        and model.endswith(".en")
        and language not in {"en", "English"}
    ):
        if language is not None:
            warnings.warn(
                f"{model} is an English-only model but receipted '{language}'; using English instead."
            )
        language = "en"

    return language


def main():
    args = CommandLine().read_command_line()
    output_dir: str = args.pop("output_dir")
    output_format: str = args.pop("output_format")
    os.makedirs(output_dir, exist_ok=True)
    model: str = args.pop("model")
    threads: int = args.pop("threads")
    language: str = args.pop("language")
    task: str = args.pop("task")
    device: str = args.pop("device")
    compute_type: str = args.pop("compute_type")
    verbose: bool = args.pop("verbose")
    model_directory: str = args.pop("model_directory")
    cache_directory: str = args.pop("model_dir")
    device_index: Union[int, List[int]] = args.pop("device_index")
    live_transcribe: bool = args.pop("live_transcribe")
    audio: str = args.pop("audio")
    local_files_only: bool = args.pop("local_files_only")
    live_volume_threshold: float = args.pop("live_volume_threshold")
    live_input_device: int = args.pop("live_input_device")
    hf_token = args.pop("hf_token")
    speaker_name = args.pop("speaker_name")

    language = get_language(language, model_directory, model)
    options = get_transcription_options(args)

    if not live_transcribe and len(audio) == 0:
        sys.stderr.write("You need to specify one or more audio files\n")
        sys.stderr.write(
            "Use `whisper-ctranslate2 --help` to see the available options.\n"
        )
        return

    word_options = [
        "highlight_words",
        "max_line_count",
        "max_line_width",
        "max_words_per_line",
    ]
    if not options.word_timestamps:
        for option in word_options:
            if args[option]:
                sys.stderr.write(f"--{option} requires --word_timestamps True\n")
                return

    if options.hallucination_silence_threshold and not options.word_timestamps:
        sys.stderr.write(
            "--hallucination_silence_threshold requires --word_timestamps True"
        )
        return

    if args["max_line_count"] and not args["max_line_width"]:
        warnings.warn("--max_line_count has no effect without --max_line_width")

    if args["max_words_per_line"] and args["max_line_width"]:
        warnings.warn("--max_words_per_line has no effect with --max_line_width")

    writer_options = list(word_options)
    writer_options.append("pretty_json")
    writer_args = {arg: args.pop(arg) for arg in writer_options}

    if not verbose and options.print_colors:
        sys.stderr.write("You cannot disable verbose and enable print colors\n")
        return

    if live_transcribe and not Live.is_available():
        Live.force_not_available_exception()

    if verbose and not language:
        if live_transcribe:
            print(
                "Consider specifying the language using `--language`. It improves significantly prediction in live transcription."
            )
        else:
            print(
                "Detecting language using up to the first 30 seconds. Use `--language` to specify the language"
            )

    if options.print_colors and output_dir and not options.word_timestamps:
        print(
            "Print colors requires word-level time stamps. Generated files in output directory will have word-level timestamps"
        )

    output_dir = os.path.abspath(output_dir)
    if model_directory:
        model_filename = os.path.join(model_directory, "model.bin")
        if not os.path.exists(model_filename):
            sys.stderr.write(f"Model file '{model_filename}' does not exists\n")
            return
        model_dir = model_directory
    else:
        model_dir = model

    if live_transcribe:
        Live(
            model_dir,
            cache_directory,
            local_files_only,
            task,
            language,
            threads,
            device,
            device_index,
            compute_type,
            verbose,
            live_volume_threshold,
            live_input_device,
            options,
        ).inference()

        return

    transcribe = Transcribe(
        model_dir,
        device,
        device_index,
        compute_type,
        threads,
        cache_directory,
        local_files_only,
    )

    diarization = len(hf_token) > 0

    if diarization:
        # Import is done here then dependencies like torch are only imported if we really need diarization
        from .diarization import Diarization

        diarization_device = "cpu" if device == "auto" else device
        diarize_model = Diarization(use_auth_token=hf_token, device=diarization_device)
        if threads > 0:
            diarize_model.set_threads(threads)

    diarization_output = {}
    if diarization:
        diarization_output = get_diarization(audio, diarize_model, verbose)

    # We need to do first the diarization of all files because CTranslate2 and torch
    # use incompatible CUDA versions and once CTranslate2 is used torch will not work
    for audio_path in audio:
        try:
            if verbose and len(audio) > 1:
                print(f"\nFile: '{audio_path} ({task})'")

            start_time = datetime.datetime.now()
            result = transcribe.inference(
                audio_path,
                task,
                language,
                verbose,
                False,
                options,
            )

            if diarization:
                if verbose:
                    print(
                        f"Time used for transcription: {datetime.datetime.now() - start_time}"
                    )
                result = diarize_model.assign_speakers_to_segments(
                    diarization_output[audio_path], result, speaker_name
                )

            writer = get_writer(output_format, output_dir)
            writer(result, audio_path, writer_args)

        except Exception as e:
            sys.stderr.write(
                f"Error: unable to process file: {audio_path} with exception '{e}'\n"
            )
            continue

    if verbose:
        print(f"Transcription results written to '{output_dir}' directory")


if __name__ == "__main__":
    main()



import unittest
import os
import json
from src.whisper_ctranslate2.writers import (
    WriteTXT,
    WriteSRT,
    WriteTSV,
    WriteVTT,
    WriteJSON,
)
from tempfile import NamedTemporaryFile
from faster_whisper.transcribe import Segment, Word


class TestWriters(unittest.TestCase):
    def _get_segment(self, text, start=0, end=1):
        return Segment(
            start=start,
            end=end,
            text=text,
            words=[],
            avg_logprob=0,
            no_speech_prob=0,
            id=0,
            seek=0,
            tokens=[],
            temperature=0,
            compression_ratio=0,
        )._asdict()

    def _read_subtitles(self, filename):
        with open(filename, "r") as fh_r:
            return fh_r.readlines()

    def _get_temp_file_name_dir(self):
        f = NamedTemporaryFile()
        return f.name, os.path.dirname(f.name)

    def test_writetext(self):
        segments = [
            self._get_segment("Hello my friends."),
            self._get_segment("How are you?"),
        ]
        results = {"text": "all text", "segments": segments}

        filename, dirname = self._get_temp_file_name_dir()
        subtitlesWriter = WriteTXT(output_dir=dirname)
        subtitlesWriter(results, filename, dict())

        r = self._read_subtitles(filename + ".txt")

        self.assertEqual(2, len(r), "len")
        self.assertEqual("Hello my friends.\n", r[0], "text")
        self.assertEqual("How are you?\n", r[1], "text")

    def test_writetext_speaker(self):
        segment = self._get_segment("Hello", start=1, end=4)
        segment["speaker"] = "John"
        segments = [segment]
        results = {"text": "all text", "segments": segments}

        filename, dirname = self._get_temp_file_name_dir()
        subtitlesWriter = WriteTXT(output_dir=dirname)
        subtitlesWriter(results, filename, dict())

        r = self._read_subtitles(filename + ".txt")

        self.assertEqual(1, len(r), "len")
        self.assertEqual("[John]: Hello\n", r[0], "text")

    def test_write_srt(self):
        segments = [
            self._get_segment("Hello my friends.", start=1, end=5),
            self._get_segment("How are you?", start=6.5, end=8),
        ]
        results = {"text": "all text", "segments": segments}

        filename, dirname = self._get_temp_file_name_dir()
        subtitlesWriter = WriteSRT(output_dir=dirname)
        subtitlesWriter(results, filename, {})
        r = self._read_subtitles(filename + ".srt")

        self.assertEqual(8, len(r), "text")
        self.assertEqual("1\n", r[0], "text")
        self.assertEqual("00:00:01,000 --> 00:00:05,000\n", r[1], "text")
        self.assertEqual("Hello my friends.\n", r[2], "text")
        self.assertEqual("\n", r[3], "text")
        self.assertEqual("2\n", r[4], "text")
        self.assertEqual("00:00:06,500 --> 00:00:08,000\n", r[5], "text")
        self.assertEqual("How are you?\n", r[6], "text")
        self.assertEqual("\n", r[7], "text")

    def test_write_tsv(self):
        segments = [
            self._get_segment("Hello my friends.", start=1, end=5),
            self._get_segment("How are you?", start=6.5, end=8),
        ]
        results = {"text": "all text", "segments": segments}

        filename, dirname = self._get_temp_file_name_dir()
        subtitlesWriter = WriteTSV(output_dir=dirname)
        subtitlesWriter(results, filename, {})
        r = self._read_subtitles(filename + ".tsv")
        self.assertEqual(3, len(r), "text")
        self.assertEqual("start\tend\ttext\n", r[0], "text")
        self.assertEqual("1000\t5000\tHello my friends.\n", r[1], "text")
        self.assertEqual("6500\t8000\tHow are you?\n", r[2], "text")

    def test_write_vtt(self):
        segments = [
            self._get_segment("Hello my friends.", start=1, end=5),
            self._get_segment("How are you?", start=6.5, end=8),
        ]
        results = {"text": "all text", "segments": segments}

        filename, dirname = self._get_temp_file_name_dir()
        subtitlesWriter = WriteVTT(output_dir=dirname)
        subtitlesWriter(results, filename, {})
        r = self._read_subtitles(filename + ".vtt")
        self.assertEqual(8, len(r), "text")
        self.assertEqual("WEBVTT\n", r[0], "text")
        self.assertEqual("\n", r[1], "text")
        self.assertEqual("00:01.000 --> 00:05.000\n", r[2], "text")
        self.assertEqual("Hello my friends.\n", r[3], "text")
        self.assertEqual("\n", r[4], "text")
        self.assertEqual("00:06.500 --> 00:08.000\n", r[5], "text")
        self.assertEqual("How are you?\n", r[6], "text")
        self.assertEqual("\n", r[7], "text")

    def test_write_srt_words(self):
        segment = self._get_segment("Hello", start=1, end=5)
        segments = [segment]
        segments[0]["words"] = [
            Word(start=1, end=2, word="Hello", probability=0)._asdict(),
        ]

        results = {"text": "all text", "segments": segments}

        filename, dirname = self._get_temp_file_name_dir()
        subtitlesWriter = WriteSRT(output_dir=dirname)
        subtitlesWriter(results, filename, {"highlight_words": True})
        r = self._read_subtitles(filename + ".srt")

        self.assertEqual(4, len(r), "text")
        self.assertEqual("1\n", r[0], "text")
        self.assertEqual("00:00:01,000 --> 00:00:02,000\n", r[1], "text")
        self.assertEqual("<u>Hello</u>\n", r[2], "text")

    def test_write_srt_words_speaker(self):
        segment = self._get_segment("Hello", start=1, end=5)
        segment["speaker"] = "John"
        segments = [segment]
        results = {"text": "all text", "segments": segments}

        filename, dirname = self._get_temp_file_name_dir()
        subtitlesWriter = WriteSRT(output_dir=dirname)
        subtitlesWriter(results, filename, {"highlight_words": True})
        r = self._read_subtitles(filename + ".srt")

        self.assertEqual(4, len(r), "text")
        self.assertEqual("1\n", r[0], "text")
        self.assertEqual("00:00:01,000 --> 00:00:05,000\n", r[1], "text")
        self.assertEqual("[John]: Hello\n", r[2], "text")

    def test_write_srt_words_max_line_width(self):
        segment = self._get_segment("Hello friends", start=1, end=5)
        segments = [segment]
        segments[0]["words"] = [
            Word(start=1, end=2, word="Hello", probability=0)._asdict(),
            Word(start=4, end=6, word="friends", probability=0)._asdict(),
        ]

        results = {"text": "all text", "segments": segments}

        filename, dirname = self._get_temp_file_name_dir()
        subtitlesWriter = WriteSRT(output_dir=dirname)
        subtitlesWriter(results, filename, {"max_line_width": 5})
        r = self._read_subtitles(filename + ".srt")
        self.assertEqual(5, len(r), "text")
        self.assertEqual("1\n", r[0], "text")
        self.assertEqual("00:00:01,000 --> 00:00:06,000\n", r[1], "text")
        self.assertEqual("Hello\n", r[2], "text")
        self.assertEqual("friends\n", r[3], "text")
        self.assertEqual("\n", r[4], "text")

    def test_write_srt_max_words_per_line(self):
        segment = self._get_segment("Hello friends", start=1, end=5)
        segments = [segment]
        segments[0]["words"] = [
            Word(start=1, end=2, word="Hello", probability=0)._asdict(),
            Word(start=4, end=6, word=" all", probability=0)._asdict(),
            Word(start=7, end=9, word=" my", probability=0)._asdict(),
            Word(start=10, end=15, word=" friends", probability=0)._asdict(),
            Word(start=16, end=17, word=" here", probability=0)._asdict(),
        ]

        results = {"text": "all text", "segments": segments}

        filename, dirname = self._get_temp_file_name_dir()
        subtitlesWriter = WriteSRT(output_dir=dirname)
        subtitlesWriter(results, filename, {"max_words_per_line": 2})
        r = self._read_subtitles(filename + ".srt")
        self.assertEqual(12, len(r), "text")
        self.assertEqual("1\n", r[0], "text")
        self.assertEqual("00:00:01,000 --> 00:00:06,000\n", r[1], "text")
        self.assertEqual("Hello all\n", r[2], "text")
        self.assertEqual("\n", r[3], "text")
        self.assertEqual("2\n", r[4], "text")
        self.assertEqual("00:00:07,000 --> 00:00:15,000\n", r[5], "text")
        self.assertEqual("my friends\n", r[6], "text")
        self.assertEqual("\n", r[7], "text")
        self.assertEqual("00:00:16,000 --> 00:00:17,000\n", r[9], "text")
        self.assertEqual("here\n", r[10], "text")
        self.assertEqual("\n", r[11], "text")

    def test_write_srt_words_max_line_count(self):
        segment = self._get_segment("Hello friends", start=1, end=5)
        segments = [segment]
        segments[0]["words"] = [
            Word(start=1, end=2, word="Hello", probability=0)._asdict(),
            Word(start=4, end=6, word="friends", probability=0)._asdict(),
        ]

        results = {"text": "all text", "segments": segments}

        filename, dirname = self._get_temp_file_name_dir()
        subtitlesWriter = WriteSRT(output_dir=dirname)
        subtitlesWriter(results, filename, {"max_line_width": 5, "max_line_count": 1})
        r = self._read_subtitles(filename + ".srt")
        self.assertEqual(8, len(r), "text")
        self.assertEqual("1\n", r[0], "text")
        self.assertEqual("00:00:01,000 --> 00:00:02,000\n", r[1], "text")
        self.assertEqual("Hello\n", r[2], "text")
        self.assertEqual("\n", r[3], "text")
        self.assertEqual("2\n", r[4], "text")
        self.assertEqual("00:00:04,000 --> 00:00:06,000\n", r[5], "text")
        self.assertEqual("friends\n", r[6], "text")
        self.assertEqual("\n", r[7], "text")

    def test_write_json(self):
        segment = self._get_segment("Hello", start=0, end=0)
        segments = [segment]

        results = {"text": "all text", "segments": segments}

        filename, dirname = self._get_temp_file_name_dir()
        subtitlesWriter = WriteJSON(output_dir=dirname)
        subtitlesWriter(results, filename, {"highlight_words": True})

        with open(filename + ".json") as json_file:
            json_data = json.load(json_file)

        self.assertEqual("all text", json_data["text"])
        self.assertEqual(1, len(json_data["segments"]))
        self.assertEqual(0, len(json_data["segments"][0]["words"]))

    def test_write_words_json(self):
        WORD_START = 1
        WORD_END = 5
        WORD = "Hello"
        segment = self._get_segment("Hello", start=0, end=0)
        segments = [segment]
        segments[0]["words"] = [
            Word(start=WORD_START, end=WORD_END, word=WORD, probability=0)._asdict(),
        ]

        results = {"text": "all text", "segments": segments}

        filename, dirname = self._get_temp_file_name_dir()
        subtitlesWriter = WriteJSON(output_dir=dirname)
        subtitlesWriter(results, filename, {"highlight_words": True})

        with open(filename + ".json") as json_file:
            json_data = json.load(json_file)

        self.assertEqual("all text", json_data["text"])
        self.assertEqual(1, len(json_data["segments"]))
        self.assertEqual(1, len(json_data["segments"][0]["words"]))

        word_dict = json_data["segments"][0]["words"][0]
        self.assertEqual(WORD, word_dict["word"])
        self.assertEqual(WORD_START, word_dict["start"])
        self.assertEqual(WORD_END, word_dict["end"])


if __name__ == "__main__":
    unittest.main()



import pathlib
from setuptools import setup
import pkg_resources
import os

HERE = pathlib.Path(__file__).parent

README = (HERE / "README.md").read_text()


def read_version(fname="src/whisper_ctranslate2/version.py"):
    exec(compile(open(fname, encoding="utf-8").read(), fname, "exec"))
    return locals()["__version__"]


setup(
    name="whisper-ctranslate2",
    version=read_version(),
    description="Whisper command line client that uses CTranslate2 and faster-whisper",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/Softcatala/whisper-ctranslate2",
    author="Jordi Mas",
    author_email="jmas@softcatala.org",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
    packages=["src/whisper_ctranslate2"],
    include_package_data=True,
    install_requires=[
        str(r)
        for r in pkg_resources.parse_requirements(
            open(os.path.join(os.path.dirname(__file__), "requirements.txt"))
        )
    ],
    extras_require={
        "dev": ["flake8==7.*", "black==24.*", "nose2"],
    },
    entry_points={
        "console_scripts": [
            "whisper-ctranslate2=src.whisper_ctranslate2.whisper_ctranslate2:main",
        ]
    },
)



import unittest
from src.whisper_ctranslate2.diarization import Diarization


class TestDiarization(unittest.TestCase):
    class Segment:
        start: float = 0.0
        end: float = 0.0

        def __init__(self, start, end):
            self.start = start
            self.end = end

    def test_no_speaker(self):
        pyannote_data = [(TestDiarization.Segment(10, 20), "A", "SPEAKER_00")]

        segment = {"start": 1, "end": 5}
        segments = [segment]

        segments = Diarization()._do_assign_speakers_to_segments(
            pyannote_data, {"segments": segments}, None
        )

        self.assertFalse("speaker" in segment)

    def test_single_speaker(self):
        pyannote_data = [(TestDiarization.Segment(2, 4), "A", "SPEAKER_00")]

        segment = {"start": 1, "end": 5}
        segments = [segment]

        segments = Diarization()._do_assign_speakers_to_segments(
            pyannote_data, {"segments": segments}, None
        )

        self.assertEqual("SPEAKER_00", segment["speaker"])

    def test_two_speakers(self):
        pyannote_data = [
            (TestDiarization.Segment(1, 5), "A", "SPEAKER_00"),
            (TestDiarization.Segment(5, 7), "B", "SPEAKER_01"),
        ]

        segment = {"start": 4, "end": 10}
        segments = [segment]

        segments = Diarization()._do_assign_speakers_to_segments(
            pyannote_data, {"segments": segments}, None
        )

        self.assertEqual("SPEAKER_01", segment["speaker"])

    def test_single_speaker_with_speakername(self):
        SPEAKER_NAME = "PARLANT"
        pyannote_data = [(TestDiarization.Segment(2, 4), "A", "SPEAKER_00")]

        segment = {"start": 1, "end": 5}
        segments = [segment]

        segments = Diarization()._do_assign_speakers_to_segments(
            pyannote_data, {"segments": segments}, SPEAKER_NAME
        )

        self.assertEqual("PARLANT_00", segment["speaker"])


if __name__ == "__main__":
    unittest.main()
